--------------------------------------------------------------------------------------
-- PureQuery Repository Lite setup script for DB2 Z---------------------------------------
---
--  Please follow below guidelines while making any updates to this script
--
--  You can change names of the following items in the script to create a repository:
---The database name. The database name is specified in the IN clause of the CREATE TABLESPACE statements.
-- The storage group ID. The storage group ID is specified in the USING STOGROUP clause of CREATE statements.
-- The table space names. The table space names are specified in the IN clause of the CREATE TABLES statements.
-- The buffer pool ID. The buffer pool ID is specified in the BUFFERPOOL clause and INDBP clause.
--
-- The table names, view names, and their schema definitions.
-----------------------------------------------------------------------------------------

-- <ScriptOptions statementTerminator=";" />

CREATE DATABASE PDQDSN
 BUFFERPOOL BP0
 INDEXBP BP0
 STOGROUP SYSDEFLT
 CCSID UNICODE;


 -- This tablespace will be used for METADATASOURCE_T
CREATE  TABLESPACE PDQTSMS
 IN PDQDSN
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 BUFFERPOOL BP0
 CCSID UNICODE
 LOCKMAX SYSTEM
 LOCKSIZE ROW
 MAXROWS 255;
 
 -- This tablespace will be used for CAPTUREDATA_T
CREATE  TABLESPACE PDQTSCD
 IN PDQDSN
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 BUFFERPOOL BP0
 CCSID UNICODE
 LOCKMAX SYSTEM
 LOCKSIZE ROW
 MAXROWS 255;
 
 -- This tablespace will be used for METADATAGROUP_T
CREATE  TABLESPACE PDQTSMG
 IN PDQDSN
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 BUFFERPOOL BP0
 CCSID UNICODE
 LOCKMAX SYSTEM
 LOCKSIZE ROW
 MAXROWS 255;
 
 -- This tablespace will be used for METADATAGROUPMEMBERS_T
CREATE  TABLESPACE PDQTSMGM
 IN PDQDSN
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 BUFFERPOOL BP0
 CCSID UNICODE
 LOCKMAX SYSTEM
 LOCKSIZE ROW
 MAXROWS 255;
 
 -- This tablespace will be used for CONTACT_T
CREATE  TABLESPACE PDQTSCT
 IN PDQDSN
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 BUFFERPOOL BP0
 CCSID UNICODE
 LOCKMAX SYSTEM
 LOCKSIZE ROW
 MAXROWS 255;

-- This tablespace will be used for SCHEMA_VERSION_T
CREATE  TABLESPACE PDQTSSV
 IN PDQDSN
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 BUFFERPOOL BP0
 CCSID UNICODE
 LOCKMAX SYSTEM
 LOCKSIZE ROW
 MAXROWS 255;
 
-- This lob tablespace will be used in auxiliary table of CAPTUREDATA_T to store the lob data 
CREATE LOB TABLESPACE PDQBTSCD
 IN PDQDSN
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 BUFFERPOOL BP0;
 
-- This lob tablespace will be used in auxiliary table of METADATASOURCE_T to store the lob data 
CREATE LOB TABLESPACE PDQBTSMS
 IN PDQDSN
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 BUFFERPOOL BP0;

CREATE TABLE METADATASOURCE_T (
  METADATASRC_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (
   START WITH 1
   INCREMENT BY 1
   MINVALUE 1
   MAXVALUE 2147483647
   NO CYCLE
   CACHE 20
   NO ORDER ),
   SOURCE VARCHAR(255) FOR MIXED DATA ,
  IMPORT_TIME TIMESTAMP NOT NULL WITH DEFAULT ,
  CONTENT_LENGTH INTEGER ,
  CONTENT BLOB(1G) ,
  SOURCEFILE VARCHAR(2048) FOR MIXED DATA 
 )
 IN PDQDSN.PDQTSMS
 AUDIT NONE
 DATA CAPTURE NONE 
 CCSID UNICODE;

-- This table holds the lob data of CONTENT column of METADATASOURCE_T table
CREATE AUXILIARY TABLE METADCONTEDAXLY_T
 IN PDQDSN.PDQBTSMS
 STORES METADATASOURCE_T
 COLUMN CONTENT;
 
 CREATE UNIQUE INDEX METADCONTEDAXLY_IDX
 ON METADCONTEDAXLY_T
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 PCTFREE 10
 BUFFERPOOL BP0     
 PIECESIZE 4194304 K;
 
-- Hold information about the metadata source files
-- Each row is one source file that was loaded.
CREATE VIEW METADATASOURCE AS SELECT
		METADATASRC_KEY,                      -- Key for this source entry
		SOURCE,                               -- Type of file:  'Analysis', 'PureQuery', etc
		IMPORT_TIME,                          -- Time file was imported
		CONTENT_LENGTH,                       -- The uncompressed length of the content
		CONTENT,                              -- Content of the file
		SOURCEFILE                            -- File path & name
		FROM METADATASOURCE_T;   


CREATE TABLE CAPTUREDATA_T (
  CAPTUREDATA_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (
   START WITH 1
   INCREMENT BY 1
   MINVALUE 1
   MAXVALUE 2147483647
   NO CYCLE
   CACHE 20
   NO ORDER ),
  GROUP_NAME VARCHAR(255) FOR MIXED DATA NOT NULL,
  GROUP_VERSION VARCHAR(255) FOR MIXED DATA ,
  UPDATE_TIME TIMESTAMP NOT NULL WITH DEFAULT ,
  CONTENT BLOB(1G) ,
  CONTENT_LENGTH INTEGER ,
  STATE VARCHAR(255) FOR MIXED DATA ,
  STATE_TRANSITION_TIME TIMESTAMP 
 )
 IN PDQDSN.PDQTSCD
 AUDIT NONE
 DATA CAPTURE NONE 
 CCSID UNICODE;
 
-- This table holds the lob data of CONTENT column of CAPTUREDATA_T table
CREATE AUXILIARY TABLE CAPTUCONTEDAXLY_T
 IN PDQDSN.PDQBTSCD
 STORES CAPTUREDATA_T
 COLUMN CONTENT;

 CREATE UNIQUE INDEX CAPTUCONTEDAXLY_IDX
 ON CAPTUCONTEDAXLY_T
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 PCTFREE 10
 BUFFERPOOL BP0     
 PIECESIZE 4194304 K;

-- Holds the content of incrementally captured data files
CREATE VIEW CAPTUREDATA AS SELECT
		CAPTUREDATA_KEY,                      -- Key 
		GROUP_NAME,                           -- Maps to NAME in METADATAGROUPS_T table
		GROUP_VERSION,                        -- Maps to VERSION in METADATAGROUPS_T table
		UPDATE_TIME,                          -- Time file was last updated
		CONTENT,                              -- Content of the file
		CONTENT_LENGTH,                       -- Uncompressed length of the content
		STATE,                                -- The current state of the data
		STATE_TRANSITION_TIME                 -- The time the current state was set
		FROM CAPTUREDATA_T;    
		

CREATE TABLE METADATAGROUP_T (
  METADATAGROUP_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (
   START WITH 1
   INCREMENT BY 1
   MINVALUE 1
   MAXVALUE 2147483647
   NO CYCLE
   CACHE 20
   NO ORDER ),
  NAME VARCHAR(255) FOR MIXED DATA NOT NULL,
  VERSION VARCHAR(255) FOR MIXED DATA ,
  ACTIVE CHAR(1) FOR MIXED DATA ,
  TYPE_ CHAR(1) FOR MIXED DATA ,
  CONTACT_KEY INTEGER, 
-- The following was added in 2.2.7; don't include in VIEW METADATAGROUP.
  ASYNC_REFRESH_TRACK INTEGER
 )
 IN PDQDSN.PDQTSMG
  AUDIT NONE
 DATA CAPTURE NONE 
 CCSID UNICODE;
 
-- The definition of all the groups. ID + VERSION must be unique 
CREATE VIEW METADATAGROUP AS SELECT 
        METADATAGROUP_KEY,                    -- Key for this entry      
        NAME,                                 -- The external ID
        VERSION,                              -- The version of this ID
        ACTIVE,                               -- 'Y' or 'N'
        TYPE_,                                -- 'S' for SQLGroups, 'R' for RuntimeGroups
        CONTACT_KEY                           -- Key into CONTACT table for any contact information.
        FROM METADATAGROUP_T;
        

CREATE TABLE METADATAGROUPMEMBERS_T (
  METADATAGROUP_KEY INTEGER NOT NULL,
  MEMBER_TYPE CHAR(1) FOR MIXED DATA NOT NULL,
  METADATASRC_KEY INTEGER ,
  CHILDGROUP_KEY INTEGER ,
  CHILDGROUP_NAME VARCHAR(255) FOR MIXED DATA ,
  CHILDGROUP_TYPE CHAR(1) FOR MIXED DATA 
 )
 IN PDQDSN.PDQTSMGM
 AUDIT NONE
 DATA CAPTURE NONE 
 CCSID UNICODE;
 
--  Holds the list of group members
--  Each row is one member of the group.  Although the row has nullable keys, one is always
--  non null and the others null depending on what type of member is in the group. 
--     If the member is a file, then METADATASRC_KEY is not null.
--     If the member is another specific group/version, then CHILDGROUP_KEY is not null.
--     If the member are all versions of a specific group, then CHILDGROUP_NAME and CHILDGROUP_TYPE are not null.
CREATE VIEW METADATAGROUPMEMBERS AS SELECT 
        METADATAGROUP_KEY,                    -- Key into METADATAGROUP_T table, holds what group id of this member.
        MEMBER_TYPE,                          -- 'C' for capture.pdqxml files, 'B' for bindProps, 'G' for genProps, 'P' for pdq.properties, 'S' for sql group  
        METADATASRC_KEY,                      -- Key into METADATASRC_T table, holds what file is part of the group.
        CHILDGROUP_KEY,                       -- Key into METADATAGROUP_T table, holds what child group is part of the group.
        CHILDGROUP_NAME,                      -- Name into METADATAGROUP_T table, holds the name of the child group.
        CHILDGROUP_TYPE                       -- Type into METADATAGROUP_T table.  Used with CHILDGROUP_NAME
        FROM METADATAGROUPMEMBERS_T;


CREATE TABLE CONTACT_T (
  CONTACT_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (
   START WITH 1
   INCREMENT BY 1
   MINVALUE 1
   MAXVALUE 2147483647
   NO CYCLE
   CACHE 20
   NO ORDER ),
  CONTACT VARCHAR(255) FOR MIXED DATA 
 )
 IN PDQDSN.PDQTSCT
 AUDIT NONE
 DATA CAPTURE NONE 
 CCSID UNICODE;
 
-- Information about contacts responsible for the metadata
CREATE VIEW CONTACT AS SELECT 
        CONTACT_KEY,                          -- Table key
        CONTACT                               -- Information about the contact
        FROM CONTACT_T;

	
CREATE TABLE SCHEMA_VERSION_T (
  COMPONENT VARCHAR(255) FOR MIXED DATA ,
  VERSION INTEGER ,
  RELEASE INTEGER ,
  MOD     INTEGER,
-- The following were added in 2.2.7; don't include in VIEW SCHEMA_VERSION.
  UUID_MOST_SIGNIFICANT_BITS BIGINT,
  UUID_LEAST_SIGNIFICANT_BITS BIGINT		
 )
 IN PDQDSN.PDQTSSV
 AUDIT NONE
 DATA CAPTURE NONE 
 CCSID UNICODE;
 

-- Metadata schema information	
-- Each row holds information about one specific component.
CREATE VIEW SCHEMA_VERSION AS SELECT
	COMPONENT,                                  -- Name of the component.  Ex: com.ibm.pureQuery
	VERSION,                                    -- Version #.  Ex for v1.2.3, this is 1
	RELEASE,                                    -- Release #   Ex for v1.2.3, this is 2
	MOD                                         -- Modification #  Ex for v1.2.3, this is 3
    FROM SCHEMA_VERSION_T;
	
	
-- According to DB2 for z/OS V8 documentation
-- indexes will automatically be dropped when the table is dropped,
-- so the generic db2LiteRepositoryDrop will drop primary key index
--
CREATE UNIQUE INDEX METADATASRC_T_PK
 ON METADATASOURCE_T
 (METADATASRC_KEY  ASC)
 NOT CLUSTER
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 PCTFREE 10
 BUFFERPOOL BP0; 

ALTER TABLE METADATASOURCE_T ADD CONSTRAINT
METADATASRC_T_PK PRIMARY KEY (METADATASRC_KEY);

CREATE UNIQUE INDEX MDGROUP_T_PK
 ON METADATAGROUP_T
 (METADATAGROUP_KEY  ASC)
 NOT CLUSTER
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 PCTFREE 10
 BUFFERPOOL BP0;

ALTER TABLE METADATAGROUP_T ADD CONSTRAINT MDGROUP_T_PK
PRIMARY KEY (METADATAGROUP_KEY);

CREATE UNIQUE INDEX CONTACT_T_PK
 ON CONTACT_T
 (CONTACT_KEY  ASC)
 NOT CLUSTER
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 PCTFREE 10
 BUFFERPOOL BP0;

ALTER TABLE CONTACT_T ADD CONSTRAINT CONTACT_T_PK PRIMARY
KEY  (CONTACT_KEY);

CREATE UNIQUE INDEX CAPTUREDATA_T_PK
 ON CAPTUREDATA_T
 (CAPTUREDATA_KEY  ASC)
 NOT CLUSTER
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 PCTFREE 10
 BUFFERPOOL BP0;
ALTER TABLE CAPTUREDATA_T ADD CONSTRAINT CAPTUREDATA_T_PK
PRIMARY KEY (CAPTUREDATA_KEY);

ALTER TABLE METADATAGROUP_T ADD CONSTRAINT
MDG_CONTACT_T_FK FOREIGN KEY
 (CONTACT_KEY)
 REFERENCES CONTACT_T
 (CONTACT_KEY)
 ON DELETE CASCADE;
	
ALTER TABLE METADATAGROUPMEMBERS_T ADD CONSTRAINT
MDGM_MDG_2_T_FK FOREIGN KEY
 (CHILDGROUP_KEY)
 REFERENCES METADATAGROUP_T
 (METADATAGROUP_KEY)
 ON DELETE CASCADE;

ALTER TABLE METADATAGROUPMEMBERS_T ADD CONSTRAINT
MDGM_MDG_T_FK FOREIGN KEY
 (METADATAGROUP_KEY)
 REFERENCES METADATAGROUP_T
 (METADATAGROUP_KEY)
 ON DELETE CASCADE;

ALTER TABLE METADATAGROUPMEMBERS_T ADD CONSTRAINT
MDGM_MDS_T_FK FOREIGN KEY
 (METADATASRC_KEY)
 REFERENCES METADATASOURCE_T
 (METADATASRC_KEY)
 ON DELETE CASCADE;

    
CREATE UNIQUE INDEX METADATAGROUP_T_IDX 
ON METADATAGROUP_T (NAME, VERSION, TYPE_) 
NOT CLUSTER
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 PCTFREE 10
 BUFFERPOOL BP0;;
    
INSERT INTO SCHEMA_VERSION_T (COMPONENT, VERSION, RELEASE, MOD) values ('com.ibm.pureQuery.lite', 2, 2, 5);
	
{upgrades};
	
commit work;
