--------------------------------------------------------------------------------------
-- PureQuery Repository upgrade script for DB2/Z -----------------------------------
--------------------------------------------------------------------------------------

-- <ScriptOptions statementTerminator=";" />

 -- This tablespace will be used for LOG_T
CREATE  TABLESPACE PDQTSLOG
 IN PDQDSN
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 BUFFERPOOL BP0
 CCSID UNICODE
 LOCKMAX SYSTEM
 LOCKSIZE ROW;

-- This lob tablespace will be used in auxiliary table of LOG_T to store the lob data 
CREATE LOB TABLESPACE PDQBTSLG
 IN PDQDSN
 USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 BUFFERPOOL BP0;

CREATE TABLE LOG_T (
        LOG_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
		GROUP_NAME VARCHAR(255),
		GROUP_VERSION VARCHAR(255), 
		HOSTIP VARCHAR(255), 
		LOG_TIME TIMESTAMP,
		LOG_MSG VARCHAR(2048),
		LOG_LEVEL VARCHAR(8),
		LOGGER_ID INTEGER,
		DUMP CLOB(1G)
	) 
 IN PDQDSN.PDQTSLOG
 AUDIT NONE
 DATA CAPTURE NONE 
 CCSID UNICODE;	
 
CREATE VIEW LOG AS SELECT
		LOG_KEY,                      -- Key for this log entry
		GROUP_NAME,                   -- Maps to NAME in METADATAGROUPS_T table
		GROUP_VERSION,                -- Maps to VERSION in METADATAGROUPS_T table
		HOSTIP,						  -- Hostname or IP address of the client that inserted this record 
		LOG_TIME,                     -- The time log message was created on the client
		LOG_MSG,                      -- Log message
		LOG_LEVEL,					  -- Log level
		LOGGER_ID,			  		  -- An ID that identifies the process that sent the log messages (is not necessarily process id).	
		DUMP						  -- Stack trace for single error, or a dump of multiple log messages
		FROM LOG_T;   		  
		
CREATE UNIQUE INDEX LOG_IDX ON LOG_T(LOG_KEY);
 		
ALTER TABLE LOG_T ADD CONSTRAINT LOG_T_PK PRIMARY KEY(LOG_KEY);
		
CREATE INDEX LOG_T_LOG_TIME_IDX ON LOG_T (LOG_TIME);

CREATE AUXILIARY TABLE LOGAUX_T
 IN PDQDSN.PDQBTSLG
 STORES LOG_T
 COLUMN DUMP;
		
CREATE UNIQUE INDEX LOGAUX_IDX ON LOGAUX_T
USING STOGROUP SYSDEFLT
   PRIQTY -1
   SECQTY -1
 PCTFREE 10
 BUFFERPOOL BP0     
 PIECESIZE 4194304 K;
		
UPDATE SCHEMA_VERSION_T SET VERSION=2, RELEASE=2, MOD=6 WHERE COMPONENT like 'com.ibm.pureQuery%';

