-- <ScriptOptions statementTerminator=";" />

CREATE TABLE STMT_T (
        STMT_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
		EXPRESSION CLOB(2097152),
		QUERYTEXT CLOB(2097152),
		QUERYTEXTTYPE CHAR(1),
		STATEMENTTYPE CHAR(6),
		EXPRESSIONTYPE CHAR(1),
		PROCESSEDSQLTEXT CLOB(2097152),
		TOTALCOST DOUBLE,
		CARDINALITY BIGINT,
		JOINCOUNT INTEGER,
        SECTIONNUM INTEGER,
        ID VARCHAR(255),
		TBSCANCOUNT INTEGER,
		IXSCANCOUNT INTEGER,
		DBPKG_KEY INTEGER,
		IS_BINDABLE CHAR(1),
		DEFAULTSCHEMA VARCHAR(255),
		PATH VARCHAR(255),
		SPECREG CLOB(2097152),
		EXECOUNT INTEGER,
		LASTUSEDTS VARCHAR(255)
	);

-- View STMT accesses the query text statements and the expressions where those statements originated.
-- Each row is one individual statement (may be SQL, JPQL, HQL, etc)
CREATE VIEW STMT AS SELECT
		STMT_KEY,                   -- Key for this statement
		EXPRESSION,                 -- Expression when query text originated
		QUERYTEXT,                  -- The query text
		QUERYTEXTTYPE,              -- Type of text.  'S' = sql
		STATEMENTTYPE,              -- Type of stmt.  'INSERT', 'SELECT', etc
		EXPRESSIONTYPE,             -- Type of expression.  'S' = sql, 'U' = unknown
		PROCESSEDSQLTEXT,           -- Any replacement SQL
		TOTALCOST,                  -- Cost from explain, null if not explained
		CARDINALITY,                -- Cardinatlity from explain, null if not explained
		JOINCOUNT,                  -- Joincount from explain, null if not explained
		SECTIONNUM,                 -- Section number of this statement from metadata source file
		ID,                         -- Id from metadata source file
		TBSCANCOUNT,                -- Table scan count from explain, null if not explained
		IXSCANCOUNT,                -- Index scan count from explain, null if not explained
		DBPKG_KEY,                  -- Foreign key to DBPKG table
		IS_BINDABLE,                -- 'Y' or 'N'
		DEFAULTSCHEMA,              -- Any default schema for this stmt
		PATH,                       -- Any default path for this stmt
		SPECREG,                    -- Special registers from metadata source file
		EXECOUNT,					-- Execution count from source file
		LASTUSEDTS					-- Last used timestamp from source file
        FROM STMT_T;
		
	
CREATE TABLE METADATASOURCE_T (
        METADATASRC_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
		SOURCE VARCHAR(255),
        IMPORT_TIME TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
        CONTENT_LENGTH INTEGER,
		CONTENT BLOB(1G),
		SOURCEFILE VARCHAR(2048)
);

-- Hold information about the metadata source files
-- Each row is one source file that was loaded.
CREATE VIEW METADATASOURCE AS SELECT
		METADATASRC_KEY,                      -- Key for this source entry
		SOURCE,                               -- Type of file:  'Analysis', 'PureQuery', etc
		IMPORT_TIME,                          -- Time file was imported
		CONTENT_LENGTH,                       -- The uncompressed length of the content
		CONTENT,                              -- Content of the file
		SOURCEFILE                            -- File path & name
		FROM METADATASOURCE_T;   
	
			
CREATE TABLE CAPTUREDATA_T (
        CAPTUREDATA_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
		GROUP_NAME VARCHAR(255) NOT NULL,
		GROUP_VERSION VARCHAR(255), 
        UPDATE_TIME TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
		CONTENT BLOB(1G),
		CONTENT_LENGTH INTEGER,
		STATE   VARCHAR(255),
		STATE_TRANSITION_TIME TIMESTAMP
);	

-- Holds the content of incrementally captured data files
CREATE VIEW CAPTUREDATA AS SELECT
		CAPTUREDATA_KEY,                      -- Key 
		GROUP_NAME,                           -- Maps to NAME in METADATAGROUPS_T table
		GROUP_VERSION,                        -- Maps to VERSION in METADATAGROUPS_T table
		UPDATE_TIME,                          -- Time file was last updated
		CONTENT,                              -- Content of the file
		CONTENT_LENGTH,                       -- Uncompressed length of the content
		STATE,                                -- The current state of the data
		STATE_TRANSITION_TIME                 -- The time the current state was set
		FROM CAPTUREDATA_T;    
		
CREATE TABLE METADATAGROUP_T (
        METADATAGROUP_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
        NAME        VARCHAR(255) NOT NULL,       
        VERSION     VARCHAR(255),
        ACTIVE      CHAR(1),       
        TYPE_       CHAR(1),
        CONTACT_KEY INTEGER,
-- The following was added in 2.2.7; don't include in VIEW METADATAGROUP.
        ASYNC_REFRESH_TRACK INTEGER
);

-- The definition of all the groups. ID + VERSION must be unique 
CREATE VIEW METADATAGROUP AS SELECT 
        METADATAGROUP_KEY,                    -- Key for this entry      
        NAME,                                 -- The external ID
        VERSION,                              -- The version of this ID
        ACTIVE,                               -- 'Y' or 'N'
        TYPE_,                                -- 'S' for SQLGroups, 'R' for RuntimeGroups
        CONTACT_KEY                           -- Key into CONTACT table for any contact information.
        FROM METADATAGROUP_T;
        
CREATE TABLE METADATAGROUPMEMBERS_T (
        METADATAGROUP_KEY INTEGER NOT NULL,
        MEMBER_TYPE CHAR(1) NOT NULL,  
        METADATASRC_KEY INTEGER,       
        CHILDGROUP_KEY INTEGER,
        CHILDGROUP_NAME VARCHAR(255),
        CHILDGROUP_TYPE CHAR(1)
);

--  Holds the list of group members
--  Each row is one member of the group.  Although the row has nullable keys, one is always
--  non null and the others null depending on what type of member is in the group. 
--     If the member is a file, then METADATASRC_KEY is not null.
--     If the member is another specific group/version, then CHILDGROUP_KEY is not null.
--     If the member are all versions of a specific group, then CHILDGROUP_NAME and CHILDGROUP_TYPE are not null.
CREATE VIEW METADATAGROUPMEMBERS AS SELECT 
        METADATAGROUP_KEY,                    -- Key into METADATAGROUP_T table, holds what group id of this member.
        MEMBER_TYPE,                          -- 'C' for capture.pdqxml files, 'B' for bindProps, 'G' for genProps, 'P' for pdq.properties, 'S' for sql group  
        METADATASRC_KEY,                      -- Key into METADATASRC_T table, holds what file is part of the group.
        CHILDGROUP_KEY,                       -- Key into METADATAGROUP_T table, holds what child group is part of the group.
        CHILDGROUP_NAME,                      -- Name into METADATAGROUP_T table, holds the name of the child group.
        CHILDGROUP_TYPE                       -- Type into METADATAGROUP_T table.  Used with CHILDGROUP_NAME
        FROM METADATAGROUPMEMBERS_T;
        
CREATE TABLE CONTACT_T (
        CONTACT_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
	    CONTACT VARCHAR(255)
);

-- Information about contacts responsible for the metadata
CREATE VIEW CONTACT AS SELECT 
        CONTACT_KEY,                          -- Table key
        CONTACT                               -- Information about the contact
        FROM CONTACT_T;


CREATE TABLE PROJECT_T (
    PROJECT_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
	NAME VARCHAR(255)
);		

-- Hold project information
-- Each row is one project (usually Eclipse project)
CREATE VIEW PROJECT AS SELECT
	PROJECT_KEY,                                  -- Key of this project
	NAME                                          -- Project name.
    FROM PROJECT_T;

CREATE TABLE DEPTAB_T (
		SCHEMANAME VARCHAR(255),
		TABLENAME VARCHAR(255),
		COLUMNNAME VARCHAR(255),
		STMT_KEY INTEGER NOT NULL
	);


-- Column dependency information
-- Each row is one fully qualified column name.
CREATE VIEW DEPTAB AS SELECT
	SCHEMANAME,                                  -- Name of schema, if known.  Null if not.
	TABLENAME,                                   -- Name of table, if known.  Null if not.
	COLUMNNAME,                                  -- Name of column, if known.  Null if not.
	STMT_KEY                                     -- Foreign key to STMT table pointing to which statement this depenedency is for.
    FROM DEPTAB_T;

CREATE TABLE STACK_T (
	STACK_ID INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
        METADATASRC_KEY INTEGER NOT NULL
	);

-- Information about the stacks defined in the SRCINFO_T table
-- Each row is one unique stack
CREATE VIEW STACK AS SELECT
	STACK_ID,                                    -- Key of table
        METADATASRC_KEY                             -- What metadata source file the stack came from
        FROM STACK_T;
	
CREATE TABLE SRCINFO_T (
                STACK_ID INTEGER NOT NULL,
                STACKTRACEORDER INTEGER,
		PATH VARCHAR(2048),
		VERSION INTEGER,
		LINENO INTEGER,
		CLASSNAME VARCHAR(255),
		METHODNAME VARCHAR(255),
		METHODSIGNATURE CLOB(10240),
		NATIVEMETHOD CHAR(5),
		JAVA_PKGNAME VARCHAR(255),
		LANGUAGE VARCHAR(10),
		PROJECT_KEY INTEGER,
		LABEL VARCHAR(255),
		LABEL_TYPE VARCHAR(255)
	);


-- Hold the source location (stack) information.
-- Each row is one source location (Ex:  database/Sample.java @ line 32).
-- Stacks may be made up of one or more source location.  For stack that contain mulitple source locations,
-- each row will have the same STACK_ID, but increasing STACKTRACEORDER values.
CREATE VIEW SRCINFO AS SELECT
	STACK_ID,                                      -- The ID for a stack.  Note this is not a key for this table.
        STACKTRACEORDER,                              -- 0 based stack order.
	PATH,                                          -- Path of entry.  Ex  database/Sample.java
	VERSION,                                       -- Version of file.  Not currently used.
	LINENO,                                        -- Line number
	CLASSNAME,                                     -- Name of the class, if known.  Ex:  Sample
	METHODNAME,                                    -- Name of the method this entry is in, if known.  Ex:  main
	METHODSIGNATURE,                               -- Signature of the method, if known.
	NATIVEMETHOD,                                  -- Is this a java native method
	JAVA_PKGNAME,                                  -- Name of the java package.  Ex:  database
	LANGUAGE,                                      -- Name of the language.  Ex: java
	PROJECT_KEY,                                   -- Foreign key to the project table, if this entry is in an eclipse project.
	LABEL,
	LABEL_TYPE FROM SRCINFO_T;
	
CREATE TABLE DBINFO_T (
		DBPKG_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
		DBPKGROOT VARCHAR(255),
		COLLECTIONID VARCHAR(255),
		CONTOKEN VARCHAR(255),
		VERSION VARCHAR(255),
		IS_BINDABLE CHAR(1)
	);


-- Database package information 	
-- Each row hold information about one package
CREATE VIEW DBINFO AS SELECT
	DBPKG_KEY,                                   -- Table key
	DBPKGROOT,                                   -- Root name of the package
	COLLECTIONID,                                -- Collection id of the package
	CONTOKEN,                                    -- Consistency token
	VERSION,                                     -- Package version
	IS_BINDABLE                                 -- 'Y' or 'N'.
	FROM DBINFO_T;
	
CREATE TABLE SCHEMA_VERSION_T (
		COMPONENT VARCHAR(255),
		VERSION INTEGER,
		RELEASE INTEGER,
		MOD		INTEGER,
-- The following were added in 2.2.7; don't include in VIEW SCHEMA_VERSION.
		UUID_MOST_SIGNIFICANT_BITS BIGINT,
        UUID_LEAST_SIGNIFICANT_BITS BIGINT		
	);

-- Metadata schema information	
-- Each row holds information about one specific component.
CREATE VIEW SCHEMA_VERSION AS SELECT
	COMPONENT,                                  -- Name of the component.  Ex: com.ibm.pureQuery
	VERSION,                                    -- Version #.  Ex for v1.2.3, this is 1
	RELEASE,                                    -- Release #   Ex for v1.2.3, this is 2
	MOD                                         -- Modification #  Ex for v1.2.3, this is 3
    FROM SCHEMA_VERSION_T;
	
CREATE TABLE SRC_STMT_T (
		STMT_KEY INTEGER NOT NULL,
		STMT_OP VARCHAR(255),
		STACK_ID INTEGER NOT NULL,
                APP_KEY  INTEGER
	);


-- Relationship table holding relationships between the statements and source locations
-- From this table you can find from what locations a specific statement was used, and how
-- it was used (prepare, execute).   	
-- Each row defines one relationship between a STMT and a specific STACK.
CREATE VIEW SRC_STMT AS SELECT
	STMT_KEY,                                    -- Foreign key to STMT
	STMT_OP,                                     -- Type of relationship.  'E' = sql execute, 'P'= sql Prepare
	STACK_ID,                                    -- Foreign key to STACK
        APP_KEY                                       -- Foreign key to application
        FROM SRC_STMT_T;
	
CREATE TABLE APP_T (
        APP_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
		NAME VARCHAR(255),
		VERSION VARCHAR(255)
);

-- Application information table hold the set of defined applications.
-- Each row is a one specific application/version combo.
CREATE VIEW APP AS SELECT
	APP_KEY,                                     -- Table key
	NAME,                                        -- Name of the application.
	VERSION                                      -- Version of the application, or null.
    FROM APP_T;



CREATE TABLE METADATASOURCE_STMT_T (
	METADATASRC_KEY INTEGER NOT NULL,
	STMT_KEY INTEGER,
        APP_KEY INTEGER,
        PROJECT_KEY INTEGER
);

-- Relationship table between the STMT table and the METADATA_SOURCE table.
-- Each row defines what statements were in which metadata files for which applications/projects.
CREATE VIEW METADATASOURCE_STMT AS SELECT
	METADATASRC_KEY,                            -- Foreign key to metadata_source table (which source file)
	STMT_KEY,                                   -- Foreign key to stmt table (which statement)
	APP_KEY,                                    -- Foreign key to application (which app this is a part of)
	PROJECT_KEY                                 -- Foreign key to project table (which project this is a part of)
    FROM METADATASOURCE_STMT_T;

ALTER TABLE STMT_T ADD CONSTRAINT STMT_T_PK PRIMARY KEY (STMT_KEY);

ALTER TABLE DBINFO_T ADD CONSTRAINT DBINFO_T_PK PRIMARY KEY (DBPKG_KEY);

ALTER TABLE PROJECT_T ADD CONSTRAINT PROJECT_T_PK PRIMARY KEY (PROJECT_KEY);

ALTER TABLE APP_T ADD CONSTRAINT APP_T_PK PRIMARY KEY(APP_KEY);

ALTER TABLE METADATASOURCE_T ADD CONSTRAINT METADATASRC_T_PK PRIMARY KEY(METADATASRC_KEY);

ALTER TABLE STACK_T ADD CONSTRAINT STACK_T_PK PRIMARY KEY (STACK_ID);

ALTER TABLE METADATAGROUP_T ADD CONSTRAINT MDGROUP_T_PK PRIMARY KEY(METADATAGROUP_KEY);

ALTER TABLE CONTACT_T ADD CONSTRAINT CONTACT_T_PK PRIMARY KEY(CONTACT_KEY);

ALTER TABLE CAPTUREDATA_T ADD CONSTRAINT CAPTUREDATA_T_PK PRIMARY KEY(CAPTUREDATA_KEY);

ALTER TABLE DEPTAB_T ADD CONSTRAINT DEPTAB_T_STMT_FK FOREIGN KEY (STMT_KEY)
	REFERENCES STMT_T (STMT_KEY)
	ON DELETE CASCADE;

ALTER TABLE STMT_T ADD CONSTRAINT DBINFO_STMT_T_FK FOREIGN KEY (DBPKG_KEY)
	REFERENCES DBINFO_T (DBPKG_KEY)
	ON DELETE CASCADE;

ALTER TABLE SRCINFO_T ADD CONSTRAINT SRCINFO_PROJ_T_FK FOREIGN KEY (PROJECT_KEY)
    REFERENCES PROJECT_T (PROJECT_KEY)
    ON DELETE CASCADE;

ALTER TABLE STACK_T ADD CONSTRAINT STACK_MDSRC_T_FK FOREIGN KEY (METADATASRC_KEY)
    REFERENCES METADATASOURCE_T (METADATASRC_KEY)
    ON DELETE CASCADE;

ALTER TABLE SRC_STMT_T ADD CONSTRAINT SRC_STMT_STMT_T_FK FOREIGN KEY (STMT_KEY)
	REFERENCES STMT_T (STMT_KEY)
	ON DELETE CASCADE;
	
ALTER TABLE SRC_STMT_T ADD CONSTRAINT SRC_STMT_STK_T_FK FOREIGN KEY (STACK_ID)
	REFERENCES STACK_T (STACK_ID)
	ON DELETE CASCADE;
	
ALTER TABLE SRC_STMT_T ADD CONSTRAINT SRC_STMT_APP_T_FK FOREIGN KEY (APP_KEY)
	REFERENCES APP_T (APP_KEY)
	ON DELETE CASCADE;

ALTER TABLE METADATASOURCE_STMT_T ADD CONSTRAINT MD_STMT_MDSRC_T_FK FOREIGN KEY (METADATASRC_KEY)
	REFERENCES METADATASOURCE_T (METADATASRC_KEY)
	ON DELETE CASCADE;
	
ALTER TABLE METADATASOURCE_STMT_T ADD CONSTRAINT MD_STMT_STMT_T_FK FOREIGN KEY (STMT_KEY)
	REFERENCES STMT_T (STMT_KEY)
	ON DELETE CASCADE;

ALTER TABLE METADATASOURCE_STMT_T ADD CONSTRAINT MD_STMT_APP_T_FK FOREIGN KEY (APP_KEY)
	REFERENCES APP_T (APP_KEY)
	ON DELETE CASCADE;

ALTER TABLE METADATASOURCE_STMT_T ADD CONSTRAINT MD_STMT_PROJ_T_FK FOREIGN KEY (PROJECT_KEY)
	REFERENCES PROJECT_T (PROJECT_KEY)
	ON DELETE CASCADE;

ALTER TABLE SRCINFO_T ADD CONSTRAINT SRCINFO_SIDS_T_FK FOREIGN KEY (STACK_ID)
    REFERENCES STACK_T (STACK_ID)
    ON DELETE CASCADE;

ALTER TABLE METADATAGROUP_T ADD CONSTRAINT MDG_CONTACT_T_FK FOREIGN KEY (CONTACT_KEY)
    REFERENCES CONTACT_T (CONTACT_KEY)
    ON DELETE CASCADE;
    
ALTER TABLE METADATAGROUPMEMBERS_T ADD CONSTRAINT MDGM_MDG_T_FK FOREIGN KEY (METADATAGROUP_KEY)
    REFERENCES METADATAGROUP_T (METADATAGROUP_KEY)
    ON DELETE CASCADE;
    
ALTER TABLE METADATAGROUPMEMBERS_T ADD CONSTRAINT MDGM_MDG_2_T_FK FOREIGN KEY (CHILDGROUP_KEY)
    REFERENCES METADATAGROUP_T (METADATAGROUP_KEY)
    ON DELETE CASCADE;
       	
ALTER TABLE METADATAGROUPMEMBERS_T ADD CONSTRAINT MDGM_MDS_T_FK FOREIGN KEY (METADATASRC_KEY)
    REFERENCES METADATASOURCE_T (METADATASRC_KEY)
    ON DELETE CASCADE;
    
CREATE UNIQUE INDEX METADATAGROUP_T_IDX ON METADATAGROUP_T (NAME, VERSION, TYPE_);
    
CREATE UNIQUE INDEX PROJECT_T_IDX on PROJECT_T(NAME);

CREATE UNIQUE INDEX APP_T_IDX on APP_T(NAME, VERSION);    
    
INSERT INTO SCHEMA_VERSION_T (COMPONENT, VERSION, RELEASE, MOD) values ('com.ibm.pureQuery', 2, 2, 4);
	
{upgrades};
	
