-- <ScriptOptions statementTerminator="@" />

CREATE TABLE LOG_T (
        LOG_KEY NUMBER(10) NOT NULL,
		GROUP_NAME VARCHAR2(255),
		GROUP_VERSION VARCHAR2(255), 
		HOSTIP VARCHAR2(255), 
		LOG_TIME DATE,
		LOG_MSG VARCHAR2(2048),
		LOG_LEVEL VARCHAR(8),
		LOGGER_ID INTEGER,
		DUMP CLOB
	) @
	
CREATE SEQUENCE LOG_T_SEQ
MINVALUE 1
    MAXVALUE 2147483647
    START WITH 1
    INCREMENT BY 1
    CACHE 20 @
 
CREATE TRIGGER LOG_T_TRIGGER
BEFORE INSERT ON LOG_T FOR EACH ROW
BEGIN
    SELECT LOG_T_SEQ.NEXTVAL INTO :NEW.LOG_KEY
    FROM DUAL;
END; @
	
CREATE VIEW LOG AS SELECT
		LOG_KEY,                      -- Key for this log entry
		GROUP_NAME,                   -- Maps to NAME in METADATAGROUPS_T table
		GROUP_VERSION,                -- Maps to VERSION in METADATAGROUPS_T table
		HOSTIP,						  -- Hostname or IP address of the client that inserted this record 
		LOG_TIME,                     -- The time log message was created on the client
		LOG_MSG,                      -- Log message
		LOG_LEVEL,					  -- Log level
		LOGGER_ID,			  		  -- An ID that identifies the process that sent the log messages (is not necessarily process id).	
		DUMP						  -- Stack trace for single error, or a dump of multiple log messages
		FROM LOG_T @   		  

ALTER TABLE LOG_T ADD CONSTRAINT LOG_T_PK PRIMARY KEY(LOG_KEY) @
		
CREATE INDEX LOG_T_LOG_TIME_IDX ON LOG_T (LOG_TIME) @
		
UPDATE SCHEMA_VERSION_T SET VERSION=2, RELEASE=2, MOD=6 WHERE COMPONENT like 'com.ibm.pureQuery%' @

